package net.ibizsys.central.plugin.boot.core.helper;


import com.alibaba.fastjson.annotation.JSONField;
import com.alibaba.fastjson.util.TypeUtils;
import com.fasterxml.jackson.annotation.JsonProperty;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import lombok.experimental.Accessors;
import net.ibizsys.central.plugin.boot.core.filter.BaseFilter;
import org.springframework.data.annotation.Id;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

/**
 * 实体对象属性缓存类
 */
public class FieldCache {

	@Getter
	@Setter
	@NoArgsConstructor
	@AllArgsConstructor
	@Accessors(chain = true)
	public static class FieldItem{
		private String codeName;
		private String fieldName;
		private String columnName;
		private String jsonName;
		private String logicName;
		private Field field;
	}

	private static Hashtable<String, Hashtable<String,FieldItem>> cacheMap = new Hashtable<>();
	private static Hashtable<String, List<FieldItem>> cacheList = new Hashtable<>();
	private static Hashtable<String, FieldItem> cacheDEKeyField = new Hashtable<>();

	private static Object objLock1=new Object();

	/**
	 * 将实体对象中的属性存入缓存中
	 * @param
	 * @return
	 */
	public static <T> Hashtable<String,FieldItem> getFieldMap(Class<T> clazz) {
		String className=clazz.getName();
		if(className.indexOf("_$")>0) {
        	className=className.substring(0, className.lastIndexOf("_$"));
		}
		if(cacheMap.containsKey(className)) {
        	return cacheMap.get(className);
		}
		synchronized (objLock1) {
			if(cacheMap.containsKey(className)) {
        	   return cacheMap.get(className);
			}
			Hashtable<String,FieldItem> result = new Hashtable<String,FieldItem>();
			List<FieldItem> list = new ArrayList<>();

			Field[] fields=clazz.getDeclaredFields();
			for(Field field:fields){
				FieldItem item=new FieldItem().setCodeName(field.getName()).setFieldName(field.getName()).setColumnName(field.getName()).setJsonName(field.getName()).setLogicName(field.getName()).setField(field);

				JSONField jsField=field.getAnnotation(JSONField.class);
				if(!ObjectUtils.isEmpty(jsField)) {
					if(!StringUtils.isEmpty(jsField.name()))
						item.setJsonName(jsField.name());
				}
				else
				{
					JsonProperty jsonProperty=field.getAnnotation(JsonProperty.class);
					if((!ObjectUtils.isEmpty(jsonProperty))&&(!StringUtils.isEmpty(jsonProperty.value())))
						item.setJsonName(jsonProperty.value());
				}

				ApiModelProperty apiModelProperty=field.getAnnotation(ApiModelProperty.class);
				if(!ObjectUtils.isEmpty(apiModelProperty)) {
					if(!StringUtils.isEmpty(apiModelProperty.value()))
						item.setFieldName(apiModelProperty.value()).setColumnName(apiModelProperty.value());
					if(!StringUtils.isEmpty(apiModelProperty.notes()))
						item.setLogicName(apiModelProperty.notes());
				}

				if(!cacheDEKeyField.containsKey(className))
				{
					if(!ObjectUtils.isEmpty(field.getAnnotation(Id.class)))
						cacheDEKeyField.put(className,item);
				}
				list.add(item);
				result.put(item.getCodeName(),item);
				result.put(item.getFieldName(),item);
				result.put(item.getColumnName(),item);
				result.put(item.getJsonName(),item);
				result.put(item.getCodeName().toLowerCase(),item);
				result.put(item.getFieldName().toLowerCase(),item);
				result.put(item.getColumnName().toLowerCase(),item);
				result.put(item.getJsonName().toLowerCase(),item);

			}
			cacheMap.put(className, result);
			cacheList.put(className,list);
			return result;
		}
	}
	public static Hashtable<String,FieldItem> getFieldMap(String className) {
		if(className.indexOf("_$")>0) {
            className=className.substring(0, className.lastIndexOf("_$"));
		}
		if(cacheMap.containsKey(className)) {
            return cacheMap.get(className);
		}
		Class clazz = null;
		try {
			clazz = Class.forName(className);
			return getFieldMap(clazz);
		}
		catch (Exception ex) {
			cacheMap.put(className, new Hashtable<String,FieldItem>());
			return cacheMap.get(className);
		}
	}



	/**
	 * 从缓存中查询实体对象主键
	 * @param
	 * @return
	 */
	public static <T> String getKeyField(Class<T> clazz) {
		String className=clazz.getName();
		if(className.indexOf("_$")>0) {
			className=className.substring(0, className.lastIndexOf("_$"));
		}
		getFieldMap(className);
		if(cacheDEKeyField.containsKey(className)) {
			return cacheDEKeyField.get(className).getCodeName();
		}
		return "";
	}



	public static <T> String getFieldRealName(Class<T> clazz,String fieldname) {
		FieldItem field=getField(clazz,fieldname);
		if(field!=null) {
			return field.getCodeName();
		}
		return "";
	}



	public static <T> FieldItem getField(Class<T> clazz,String fieldname) {
		Hashtable<String,FieldItem> fields= getFieldMap(clazz);
		if(fields.containsKey(fieldname)) {
			return fields.get(fieldname);
		}
		else if(fields.containsKey(fieldname.toLowerCase())) {
			return fields.get(fieldname.toLowerCase());
		}
		else if(fields.containsKey(fieldname.toLowerCase().replace("_",""))) {
			return fields.get(fieldname.toLowerCase().replace("_",""));
		}
		else {
			return null;
		}
	}


	public static  <T> Object fieldValueOf(Class<T> clazz,String fieldname,Object fieldValue) {
		if(fieldValue==null)
			return null;
		Object resultValue=fieldValue;
		FieldItem item=getField(clazz,fieldname);
		if(item!=null) {
			Class<?> type=item.getField().getType();

			resultValue = TypeUtils.castToJavaBean(fieldValue,type);
		}
		return resultValue;
	}

}