package net.ibizsys.central.plugin.boot.core.service;

import net.ibizsys.central.dataentity.IDataEntityRuntime;
import net.ibizsys.central.dataentity.IDataEntityRuntimeContext;
import net.ibizsys.central.plugin.boot.core.domain.BaseData;
import net.ibizsys.central.plugin.boot.core.filter.BaseFilter;
import net.ibizsys.central.plugin.boot.core.helper.StringAdvUtils;
import net.ibizsys.central.plugin.boot.core.runtime.IBootService;
import net.ibizsys.central.plugin.boot.core.runtime.IBootSystemRuntime;
import net.ibizsys.model.dataentity.action.IPSDEAction;
import net.ibizsys.model.dataentity.ds.IPSDEDataSet;
import org.springframework.data.domain.Page;
import java.io.Serializable;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

public interface IBaseService<T extends BaseData, F extends BaseFilter> {

    String getDataEntityId();

    default IBootService getRuntimeService() {
        return null;
    }

    default void setRuntimeService(IBootService runtimeService) { }

    default IBootSystemRuntime getSystemRuntime(){
        return getRuntimeService().getSystemRuntime();
    }

    default IDataEntityRuntime getDataEntityRuntime(){
        return getRuntimeService().getDataEntityRuntime();
    }

    Method getServiceMethod(String name, Class clazz);

    default IDataEntityRuntimeContext getDataEntityRuntimeContext() {
        return getRuntimeService().getDataEntityRuntimeContext();
    }

    default Object executeAction(String strActionName, IPSDEAction iPSDEAction, Object[] args) throws Throwable {
        return this.getRuntimeService().getDataEntityRuntime().executeAction(strActionName, iPSDEAction, args, true);
    }


    default Object fetchDataSet(String strDataSetName, IPSDEDataSet iPSDEDataSet, Object[] args) throws Throwable {
        return this.getRuntimeService().getDataEntityRuntime().fetchDataSet(strDataSetName, iPSDEDataSet, args, true);
    }


    default Object executeActionReal(IDataEntityRuntimeContext iDataEntityRuntimeContext, String strActionName, IPSDEAction iPSDEAction, Object[] args, Object actionData) throws Throwable{
        return iDataEntityRuntimeContext.executeActionReal(strActionName, iPSDEAction, args, actionData);
    }

    default Object fetchDataSetReal(IDataEntityRuntimeContext iDataEntityRuntimeContext, String strDataSetName, IPSDEDataSet iPSDEDataSet, Object[] args, Object actionData) throws Throwable{
        return iDataEntityRuntimeContext.fetchDataSetReal(strDataSetName, iPSDEDataSet, args, actionData);
    }

    default void beforeAction(String strActionName,Object arg)  throws Throwable{
        Method beforeMethod = getServiceMethod("before"+ StringAdvUtils.pascalcase(strActionName), arg.getClass());
        if (beforeMethod != null)
            beforeMethod.invoke(this, arg);
    }

    default void beforeFetch(String strActionName,Object arg)  throws Throwable{
        Method beforeMethod = getServiceMethod("beforeFetch"+ StringAdvUtils.pascalcase(strActionName), arg.getClass());
        if (beforeMethod != null)
            beforeMethod.invoke(this, arg);
    }

    default void afterAction(String strActionName,Object arg)  throws Throwable{
        Method afterMethod = getServiceMethod("after"+ StringAdvUtils.pascalcase(strActionName), arg.getClass());
        if (afterMethod != null)
            afterMethod.invoke(this, arg);
    }

    default boolean create(T dto) throws Throwable {
        beforeAction("Create",dto);
        this.getRuntimeService().getDataEntityRuntime().executeAction("Create", null, new Object[]{dto}, true);
        afterAction("Create",dto);
        return true;
    }

    default boolean onCreate(T dto) throws Throwable {
        this.getDataEntityRuntimeContext().executeActionReal("Create", null, new Object[]{dto}, null);
        return true;
    }

    default boolean update(T dto) throws Throwable {
        beforeAction("Update",dto);
        this.getRuntimeService().getDataEntityRuntime().executeAction("Update", null, new Object[]{dto}, true);
        afterAction("Update",dto);
        return true;
    }

    default boolean onUpdate(T dto) throws Throwable {
        this.getDataEntityRuntimeContext().executeActionReal("Update", null, new Object[]{dto}, null);
        return true;
    }

    default boolean save(T dto) throws Throwable {
        beforeAction("Save",dto);
        this.getRuntimeService().getDataEntityRuntime().executeAction("Save", null, new Object[]{dto}, true);
        afterAction("Save",dto);
        return true;
    }

    default boolean onSave(T dto) throws Throwable {
        this.getDataEntityRuntimeContext().executeActionReal("Save", null, new Object[]{dto}, null);
        return true;
    }

    default <K extends Serializable> boolean remove(K key) throws Throwable {
        beforeAction("Remove",key);
        List keys=new ArrayList();
        keys.add(key);
        if(!remove(keys))
            return false;
        afterAction("Remove",key);
        return true;
    }

    default boolean remove(List keys) throws Throwable {
        beforeAction("Remove",keys);
        this.getRuntimeService().getDataEntityRuntime().executeAction("Remove", null, new Object[]{keys}, true);
        afterAction("Remove",keys);
        return true;
    }

    default boolean onRemove(List keys) throws Throwable {
        this.getDataEntityRuntimeContext().executeActionReal("Remove", null, new Object[]{keys}, null);
        return true;
    }

    default <K extends Serializable> T get(K key) throws Throwable {
        T dto = (T)this.getRuntimeService().getDataEntityRuntime().executeAction("Get", null, new Object[]{key}, true);
        if(dto!=null)
            afterAction("Get",dto);
        return dto;
    }

    default <K extends Serializable> T onGet(K key) throws Throwable {
        return (T)this.getDataEntityRuntimeContext().executeActionReal("Get", null, new Object[]{key}, null);
    }

    default T getDraft(T dto) throws Throwable {
        beforeAction("GetDraft",dto);
        T rtdto = (T)this.getRuntimeService().getDataEntityRuntime().executeAction("GetDraft", null, new Object[]{dto}, true);
        if(dto!=null)
            afterAction("GetDraft",rtdto);
        return rtdto;
    }

    default T onGetDraft(T dto) throws Throwable {
        return (T)this.getDataEntityRuntimeContext().executeActionReal("GetDraft", null, new Object[]{dto}, null);
    }

    default Integer checkKey(T dto) throws Throwable {
        beforeAction("CheckKey",dto);
        return  (Integer) this.getRuntimeService().getDataEntityRuntime().executeAction("CheckKey", null, new Object[]{dto}, true);
    }

    default Integer onCheckKey(T dto) throws Throwable {
        return (Integer) this.getDataEntityRuntimeContext().executeActionReal("CheckKey", null, new Object[]{dto}, null);
    }



    default Page<T> fetchDefault(F dto) throws Throwable {
        beforeFetch("Default",dto);
        return (Page) this.getRuntimeService().getDataEntityRuntime().fetchDataSet("DEFAULT", dto.getDataSet(), new Object[]{dto}, true);
    }

    default Page<T> onFetchDefault(F dto) throws Throwable {
        return (Page) this.getDataEntityRuntimeContext().fetchDataSetReal("DEFAULT", dto.getDataSet(), new Object[]{dto}, null);
    }

    default List<T> selectDefault(F dto) throws Throwable {
        return (List)this.getRuntimeService().getDataEntityRuntime().selectDataQuery("DEFAULT", dto);
    }

    default List<T> selectSimple(F dto) throws Throwable {
        return (List)this.getRuntimeService().getDataEntityRuntime().selectDataQuery("SIMPLE", dto);
    }

    default List<T> selectView(F dto) throws Throwable {
        return (List)this.getRuntimeService().getDataEntityRuntime().selectDataQuery("VIEW", dto);
    }

}
